# -*- coding: utf-8 -*-
from plone.app.upgrade.utils import loadMigrationProfile

import logging


logger = logging.getLogger(__name__)


def reload_gs_profile(context):
    loadMigrationProfile(
        context,
        'profile-collective.conferences:default',
    )



# -*- coding: utf-8 -*-
from collective.conferences import _
from plone import api
from plone.app.textfield import RichText
from plone.supermodel import model
from plone.supermodel.directives import primary
from Products.Five import BrowserView
from zope import schema


class IRoomfolder(model.Schema):
    """A folder for the rooms of a conference.
    """

    title = schema.TextLine(
        title=_(u'Name of the room folder'),
    )

    description = schema.Text(
        title=_(u'roomfolder description'),
    )

    primary('details')
    details = RichText(
        title=_(u'Information about the Conference Rooms'),
        required=False,
    )


class RoomfolderView(BrowserView):

    def canRequestReview(self):
        return api.user.has_permission('cmf.RequestReview', obj=self.context)



# -*- coding: utf-8 -*-
from plone.dexterity.content import Item


class CustomSpeakerName(Item):
    """Custom name for a release and linked release from the title and
    the release number"""

    @property
    def title(self):
        return self.firstname + ' ' + self.lastname

    def setTitle(self, value):
        return



# -*- coding: utf-8 -*-
from collective.conferences import _
from plone.app.textfield import RichText
from plone.supermodel import model
from plone.supermodel.directives import primary
from Products.Five import BrowserView
from zope import schema


class IConference(model.Schema):

    title = schema.TextLine(
        title=_(u'Conference Title'),
    )

    description = schema.Text(
        title=_(u'Conference summary'),
        required=False,
    )

    primary('details')
    details = RichText(
        title=_(u'Details'),
        description=_(u'Write information about the conference'),
        required=False,
    )


class ConferenceView(BrowserView):
    pass



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from Acquisition import aq_parent
from collective import dexteritytextindexer
from collective.conferences import _
from collective.conferences.common import endDefaultValue
from collective.conferences.common import startDefaultValue
from plone import api
from plone.app.contentlisting.interfaces import IContentListing
from plone.app.textfield import RichText
from plone.app.z3cform.widget import SelectFieldWidget
from plone.autoform import directives
from plone.autoform.directives import read_permission
from plone.autoform.directives import write_permission
from plone.namedfile.field import NamedBlobFile
from plone.supermodel import model
from plone.supermodel.directives import primary
from Products.CMFPlone.utils import safe_unicode
from Products.Five import BrowserView
from z3c.form.browser.radio import RadioFieldWidget
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope import schema
from zope.interface import directlyProvides
from zope.interface import Invalid
from zope.interface import invariant
from zope.schema.interfaces import IContextSourceBinder
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


def vocabCfPTopics(context):
    # For add forms
    catalog = api.portal.get_tool(name='portal_catalog')
    results = catalog.uniqueValuesFor('callforpaper_topics')
    terms = []
    for value in results:
        terms.append(SimpleTerm(value, token=value.encode('unicode_escape'), title=value))

    return SimpleVocabulary(terms)


directlyProvides(vocabCfPTopics, IContextSourceBinder)


class ChooseLicense(Invalid):
    __doc__ = _(safe_unicode(
        'Please choose a license for your talk.'))


# class StartBeforeEnd(Invalid):
#     __doc__ = _(u"The start or end date is invalid")


class IWorkshop(model.Schema):
    """A conference workshop. Workshops are managed inside tracks of the Program.
    """

    title = schema.TextLine(
        title=_(u'Title'),
        description=_(u'Workshop title'),
    )

    description = schema.Text(
        title=_(u'Workshop summary'),
    )

    primary('details')
    details = RichText(
        title=_(u'Workshop details'),
        required=False,
    )

    speaker = RelationList(
        title=_(u'Workshop Leader'),
        default=[],
        value_type=RelationChoice(vocabulary='ConferenceSpeaker'),
        required=False,
        missing_value=[],
    )
    directives.widget(
        'speaker',
        SelectFieldWidget,
    )

    dexteritytextindexer.searchable('call_for_paper_topics')
    directives.widget(call_for_paper_topic=RadioFieldWidget)
    call_for_paper_topic = schema.List(
        title=_(u'Choose the topic for your workshop'),
        value_type=schema.Choice(source=vocabCfPTopics),
        required=True,
    )

    directives.widget(planedworkshoplength=RadioFieldWidget)
    planedworkshoplength = schema.List(
        title=_(u'Planed Length'),
        description=_(u"Give an estimation about the time you'd plan for your workshop."),
        value_type=schema.Choice(source='WorkshopLength'),
        required=True,
    )

    directives.widget(license=RadioFieldWidget)
    license = schema.List(
        title=_(u'License Of Your Talk'),
        description=_(u'Choose a license for your talk'),
        value_type=schema.Choice(source='ContentLicense'),
        required=True,
    )

    read_permission(conferencetrack='cmf.ReviewPortalContent')
    write_permission(conferencetrack='cmf.ReviewPortalContent')
    conferencetrack = RelationList(
        title=_(u'Choose the track for this talk'),
        default=[],
        value_type=RelationChoice(vocabulary='ConferenceTrack'),
        required=False,
        missing_value=[],
    )
    directives.widget(
        'conferencetrack',
        RadioFieldWidget,
    )

    messagetocommittee = schema.Text(
        title=_(u'Messages to the Program Committee'),
        description=_(
            u'You can give some information to the committee here, e.g. about days you are (not) available to give the workshop'),
        required=False,
    )

    read_permission(workshoplength='cmf.ReviewPortalContent')
    write_permission(workshoplength='cmf.ReviewPortalContent')
    directives.widget(workshoplength=RadioFieldWidget)
    workshoplength = schema.List(
        title=_(u'Workshop Length'),
        description=_(u'Set a time frame for the workshop in minutes.'),
        value_type=schema.Choice(source='WorkshopLength'),
        required=False,
    )

    read_permission(positionintrack='cmf.ReviewPortalContent')
    write_permission(positionintrack='cmf.ReviewPortalContent')
    positionintrack = schema.Int(
        title=_(u'Position In The Track'),
        description=_(u'Choose a number for the order in the track'),
        required=False,
    )

    write_permission(startitem='collective.conferences.ModifyTalktime')
    startitem = schema.Datetime(
        title=_(u'Startdate'),
        description=_(u'Start date'),
        defaultFactory=startDefaultValue,
        required=False,
    )

    write_permission(enditem='collective.conferences.ModifyTalktime')
    enditem = schema.Datetime(
        title=_(u'Enddate'),
        description=_(u'End date'),
        defaultFactory=endDefaultValue,
        required=False,
    )

    slides = NamedBlobFile(
        title=_(u'Workshop slides / material'),
        description=_(
            u'Please upload your workshop presentation or material about the content of the workshop '
            u'in front or short after you have given the workshop.'),
        required=False,
    )

    read_permission(reviewNotes='cmf.ReviewPortalContent')
    write_permission(reviewNotes='cmf.ReviewPortalContent')
    reviewNotes = schema.Text(
        title=u'Review notes',
        required=False,
    )

    @invariant
    def validateLicensechoosen(data):
        if not data.license:
            raise ChooseLicense(
                _(safe_unicode('Please choose a license for your talk.'),
                  ),
            )


def newworkshopadded(self, event):
    if api.portal.get_registry_record(
            'plone.email_from_address') is not None:
        contactaddress = api.portal.get_registry_record(
            'plone.email_from_address')
        current_user = api.user.get_current()

        length = self.planedworkshoplength[0]
        cfp = self.call_for_paper_topic[0]
        details = self.details.output

        api.portal.send_email(
            recipient=current_user.getProperty('email'),
            sender=contactaddress,
            subject=safe_unicode('Your Workshop Proposal'),
            body=safe_unicode('You submitted a conference workshop:\n'
                              'title: {0},\n'
                              'summary: {1},\n'
                              'details: {2},\n'
                              'proposed length: {3} minutes\nfor the call for papers '
                              'topic: {4}\nwith the following message to the conference '
                              'committee: {5}\n\n'
                              'Best regards,\n'
                              'The Conference Committee').format(
                self.title, self.description, details,
                length, cfp, self.messagetocommittee),
        )


class WorkshopView(BrowserView):

    def canRequestReview(self):
        return api.user.has_permission('cmf.RequestReview', obj=self.context)

    def workshopLeaders(self):
        results = []
        for rel in self.context.speaker:
            if rel.isBroken():
                # skip broken relations
                continue
            obj = rel.to_object
            if api.user.has_permission('View', obj=obj):
                results.append(obj)
        return IContentListing(results)

    def conferenceTrack(self):
        results = []
        for rel in self.context.conferencetrack:
            if rel.isBroken():
                # skip broken relations
                continue
            obj = rel.to_object
            if api.user.has_permission('View', obj=obj):
                results.append(obj)
        return IContentListing(results)

    def WorkshopRoom(self):

        from collective.conferences.track import ITrack
        parent = aq_parent(aq_inner(self.context))
        if ITrack.providedBy(parent):
            room = parent.room.to_object.title
        else:
            room = ''
        return room



# -*- coding: utf-8 -*-
from collective.conferences import _
from plone import api
from plone.app.textfield import RichText
from plone.namedfile.field import NamedBlobImage
from plone.supermodel import model
from Products.Five import BrowserView
from zope import schema
from zope.interface import Invalid

import re


checkEmail = re.compile(
    r'[a-zA-Z0-9._%-]+@([a-zA-Z0-9-]+\.)+[a-zA-Z]{2,4}').match


def validateEmail(value):
    if not checkEmail(value):
        raise Invalid(_(u'Invalid email address'))
    return True


class IConferenceSpeaker(model.Schema):
    """A conference speaker or leader of a workshop. Speaker can be added anywhere.
    """

    lastname = schema.TextLine(
        title=_(u'Last name'),
    )

    firstname = schema.TextLine(
        title=_(u'First name'),
        required=True,
    )

    street = schema.TextLine(
        title=_(u'Street'),
        description=_(u'For those requiring visa, please add your full postal address details'),
        required=False,
    )

    city = schema.TextLine(
        title=_(u'City'),
        description=_(u'For those requiring visa, please add your full postal address details'),
        required=False,
    )

    postalcode = schema.TextLine(
        title=_(u'Postal Code'),
        description=_(u'For those requiring visa, please add your full postal address details'),
        required=False,
    )

    country = schema.TextLine(
        title=_(u'Country'),
        description=_(u'For those requiring visa, please add your full postal address details'),
        required=False,
    )

    email = schema.ASCIILine(
        title=_(u'Your email address'),
        constraint=validateEmail,
        required=True,
    )

    telephonenumber = schema.TextLine(
        title=_(u'Telephone Number'),
        description=_(
            u'Please fill in your telephone number so that we could get in contact with you by phone if necessary.'),
        required=False,
    )
    mobiletelepone = schema.TextLine(
        title=_(u'Mobile Telephone Number'),
        description=_(
            u'Please fill in your mobile telephone number so that we could get in contact with you '
            u'during the conference.'),
        required=True,
    )

    organisation = schema.TextLine(
        title=_(u'Organisation'),
        required=False,
    )

    description = schema.Text(
        title=_(u'A short bio'),
    )

    bio = RichText(
        title=_(u'Bio'),
        required=False,
    )

    picture = NamedBlobImage(
        title=_(u'Picture'),
        description=_(u'Please upload an image'),
        required=False,
    )


def notifyUser(self, event):
    user = api.user.get_current()
    sender = api.portal.get_registry_record(
        'plone.email_from_address')
    email = user.getProperty('email')

    if not sender:
        return

    subject = 'Is this you?'
    message = 'A speaker / leader of a workshop called {0} was added here {1}. If ' \
              'this is you, everything is fine.'.format(self.title, self.absolute_url())

    api.portal.send_email(
        recipient='{0}'.format(email),
        sender='{0}'.format(sender),
        subject='{0}'.format(subject),
        body='{0}'.format(message))


class ConferenceSpeakerView(BrowserView):

    def talks_of_speaker(self):
        catalog = api.portal.get_tool(name='portal_catalog')

        # execute a search
        results = catalog(speakertalk='Talk 1')
        # examine the results
        for brain in results:
            url = brain.getURL()

        return url

    def talks_of_speaker2(self):
        catalog = api.portal.get_tool(name='portal_catalog')

        # execute a search
        results = catalog(portal_type='collective.conferences.talk',
                          review_state='published')
        # examine the results
        for brain in results:
            title = brain.Title

        return title

    def speakertalks(self):

        speakername = self.context.title
        results = api.content.find(portal_type='collective.conferences.talk',
                                   review_state='published',
                                   presenters=speakername)

        objects = []

        for brain in results:
            objects.append(brain)

        return objects

    def speakerworkshops(self):

        speakername = self.context.title
        results = api.content.find(portal_type='collective.conferences.workshop',
                                   review_state='published',
                                   workshopleader=speakername)

        objects = []

        for brain in results:
            objects.append(brain)



# -*- coding: utf-8 -*-
from collective.conferences import _
from plone.autoform import directives
from plone.supermodel import model
from Products.Five import BrowserView
from zope import schema


class IWorkshopfolder(model.Schema):

    directives.mode(title='hidden')
    title = schema.TextLine(
        title=_(u'Name Of The Workshopfolder'),
        default=_(u'Workshops'),
    )

    description = schema.Text(
        title=_(u'Workshop Folder Description'),
        required=False,
    )


class WorkshopfolderView(BrowserView):

    pass



# -*- coding: utf-8 -*-
from collective.conferences import _
from DateTime import DateTime
from plone import api
from plone.app.contentlisting.interfaces import IContentListing
from plone.app.textfield import RichText
from plone.autoform import directives
from plone.supermodel import model
from plone.supermodel.directives import primary
from Products.Five import BrowserView
from z3c.form.browser.radio import RadioFieldWidget
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope import schema
from zope.interface import Invalid
from zope.interface import invariant


class StartBeforeEnd(Invalid):
    __doc__ = _(u'The start or end date is invalid')


class StartBeforeConferenceProgram(Invalid):
    __doc__ = _(u'The start of the track could not be set before the conference program.')


class EndAfterConferenceProgram(Invalid):
    __doc__ = _(u'The end of the track could not be set after the conference program.')


class ITrack(model.Schema):
    """A conference track. Tracks are managed inside Programs.
    """

    title = schema.TextLine(
        title=_(u'Title'),
        description=_(u'Track title'),
    )

    description = schema.Text(
        title=_(u'Track summary'),
    )

    primary('details')
    details = RichText(
        title=_(u'Track details'),
        required=False,
    )

    trackstart = schema.Datetime(
        title=_(u'Startdate'),
        description=_(u'Start date'),
        required=False,
    )

    trackend = schema.Datetime(
        title=_(u'Enddate'),
        description=_(u'End date'),
        required=False,
    )

    room = RelationList(
        title=_(u'Choose the room for the track'),
        default=[],
        value_type=RelationChoice(vocabulary='ConferenceRoom'),
        required=False,
        missing_value=[],
    )
    directives.widget(
        'room',
        RadioFieldWidget,
    )

    @invariant
    def validateStartEnd(data):
        if data.trackstart is not None and data.trackend is not None:
            if data.trackstart > data.trackend:
                raise StartBeforeEnd(_(
                    u'The start date must be before the end date.'))

    @invariant
    def validateStartNotBeforeProgram(data):
        if data.trackstart is not None:
            catalog = api.portal.get_tool(name='portal_catalog')
            result = catalog.uniqueValuesFor('programstart')
            trackstart = DateTime(data.trackstart).toZone('UTC')
            if DateTime(trackstart) < DateTime(result[0]):
                raise StartBeforeConferenceProgram(
                    _(u'The start date could not be set before the begin of the conference program.'))

    @invariant
    def validateEndNotAfterProgram(data):
        if data.trackend is not None:
            catalog = api.portal.get_tool(name='portal_catalog')
            result = catalog.uniqueValuesFor('programend')
            trackend = DateTime(data.trackend).toZone('UTC')
            if DateTime(trackend) > DateTime(result[0]):
                raise EndAfterConferenceProgram(
                    _(u"The end date couldn't be set after the end of the conference program."))


# def setdates(item):
#    if not item.track:
#        return
#    catalog = component.getUtility(ICatalog)
#    intids = component.getUtility(IIntIds)
#    items = [intids.queryObject(rel.from_id) for rel in catalog.findRelations({'to_id': intids.getId(item.track.to_object),
#                                                                                #'from_interfaces_flattened': ITalk
#                                                                                })]
#    start = item.track.to_object.start
#    for t in items:
#        t.startitem=start
#        t.enditem=t.startitem + datetime.timedelta(minutes=int(t.length))
#        start = t.enditem


class TrackView(BrowserView):

    def canRequestReview(self):
        return api.user.has_permission('cmf.RequestReview', obj=self.context)

    def talks_workshops(self):
        tracktitle = self.context.title
        talks_workshops = api.content.find(depth=3,
                                           portal_type=('collective.conferences.talk',
                                                        'collective.conferences.workshop',
                                                        'collective.conferences.conferencebreak'),
                                           conferencetrack=tracktitle,
                                           review_state='published',
                                           sort_on='orderintrack')
        return [x.getObject() for x in talks_workshops]

    def trackRoom(self):
        results = []
        for rel in self.context.room:
            if rel.isBroken():
                # skip broken relations
                continue
            obj = rel.to_object
            if api.user.has_permission('View', obj=obj):
                results.append(obj)
        return IContentListing(results)



# -*- coding: utf-8 -*-
from collective.conferences import _
from collective.conferences.common import yesnochoice
from plone import api
from plone.app.textfield import RichText
from plone.supermodel import model
from plone.supermodel.directives import primary
from Products.Five import BrowserView
from zope import schema


# from collective.conferences.attendee import IAttendee

class IAttendeefolder(model.Schema):
    """A attendee folder. The attendee of the conference are created in the folder.
    """

    title = schema.TextLine(
        title=_(u'Name of the attendee folder'),
    )

    description = schema.Text(
        title=_(u'attendee folder description'),
        required=False,
    )

    primary('moreinformation')
    moreinformation = RichText(
        title=_(u'Information about registration process'),
        required=False,
    )

    conferencefee = schema.Choice(
        title=_(u'Registration Fee?'),
        description=_(u'Have one to pay a registration fee?'),
        vocabulary=yesnochoice,
        required=True,
    )

    paymentoptions = schema.List(
        title=_(u'Payment Options'),
        description=_(u'Fill in one payment option per line.'),
        default=['Bank one',
                 'Bank two'],
        value_type=schema.TextLine(),
    )


class AttendeefolderView(BrowserView):

    def canRequestReview(self):
        return api.user.has_permission('cmf.RequestReview', obj=self.context)



# -*- coding: utf-8 -*-
from collective.conferences.conferencebreak import IConferencebreak
from collective.conferences.program import IProgram
from collective.conferences.talk import ITalk
from collective.conferences.workshop import IWorkshop
from DateTime import DateTime
from plone.indexer.decorator import indexer


@indexer(ITalk)
def presenternames(object, **kw):
    results = []
    for rel in object.speaker:
        if rel.isBroken():
            continue
        obj = rel.to_object.title
        results.append(obj)
    return results


@indexer(IWorkshop)
def workshopleadernames(object, **kw):
    results = []
    for rel in object.speaker:
        if rel.isBroken():
            continue
        obj = rel.to_object.title
        results.append(obj)
    return results


@indexer(IConferencebreak)
def conferencebreaktrackname(object, **kw):
    results = []
    for rel in object.conferencetrack:
        if rel.isBroken():
            continue
        obj = rel.to_object.title
        results.append(obj)
    return results


@indexer(IWorkshop)
def workshoptrackname(object, **kw):
    results = []
    for rel in object.conferencetrack:
        if rel.isBroken():
            continue
        obj = rel.to_object.title
        results.append(obj)
    return results


@indexer(ITalk)
def talktrackname(object, **kw):
    results = []
    for rel in object.conferencetrack:
        if rel.isBroken():
            continue
        obj = rel.to_object.title
        results.append(obj)
    return results


@indexer(IProgram)
def programStartIndexer(obj):
    if obj.start is None:
        return None
    programstart = DateTime(obj.start).toZone('UTC')
    return DateTime(programstart).ISO()


@indexer(IProgram)
def programEndIndexer(obj):
    if obj.end is None:
        return None
    programend = DateTime(obj.end).toZone('UTC')
    return DateTime(programend).ISO()



# -*- coding: utf-8 -*-
from collective.conferences import _
from plone.app.textfield import RichText
from plone.namedfile.field import NamedBlobImage
from plone.supermodel import model
from plone.supermodel.directives import primary
from Products.Five import BrowserView
from zope import schema


class IRoom(model.Schema):
    """A conference room.
    """

    title = schema.TextLine(
        title=_(u'Name of the Room'),
    )

    description = schema.Text(
        title=_(u'A description of the room and its location'),
    )

    picture = NamedBlobImage(
        title=_(u'A picture of the room'),
        description=_(u'Please upload an image'),
        required=False,
    )

    primary('details')
    details = RichText(
        title=_(u"A full description of the room, it's location and the way to get there"),
        required=True,
    )

    capacity = schema.Int(
        title=_(u'Capacity of the room'),
        description=_(u'Please fill in the maximum number of attendees'),
        required=False,
    )


class RoomView(BrowserView):
    pass



# -*- coding: utf-8 -*-
from collective.conferences import _
from collective.conferences.common import yesnochoice
from plone.autoform import directives
from plone.dexterity.browser import add
from plone.dexterity.browser import edit
from plone.supermodel import model
from Products.Five import BrowserView
from z3c.form.browser.radio import RadioFieldWidget
from zope import schema
from zope.interface import directlyProvides
from zope.schema.interfaces import IContextSourceBinder
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


def vocabpaymentoptions(context):
    from collective.conferences.attendeefolder import IAttendeefolder
    while context is not None and not IAttendeefolder.providedBy(context):
        # context = aq_parent(aq_inner(context))
        context = context.__parent__

    payment_list = []
    if context is not None and context.paymentoptions:
        payment_list = context.paymentoptions

    terms = []
    for value in payment_list:
        terms.append(SimpleTerm(value, token=value.encode('unicode_escape'),
                                title=value))

    return SimpleVocabulary(terms)


directlyProvides(vocabpaymentoptions, IContextSourceBinder)


class IAttendee(model.Schema):
    """A conference attendee. Attendees can be added anywhere.
    """

    paymentway = SimpleVocabulary(
        [SimpleTerm(value=u'by bank transfer', title=_(u'By Bank transfer')),
         SimpleTerm(value=u'by paypal', title=_(u'Payment by PayPal')),
         SimpleTerm(value=u'by check', title=_(u'By Check'))],
    )

    title = schema.TextLine(
        title=_(u'Firstname Lastname'),
    )

    street = schema.TextLine(
        title=_(u'Street'),
        description=_(u'This data is mandatory and required for our internal procedures'),
        required=True,
    )

    city = schema.TextLine(
        title=_(u'City'),
        description=_(u'This data is mandatory and required for our internal procidures'),
        required=True,
    )

    postalcode = schema.TextLine(
        title=_(u'Postal Code'),
        description=_(u'This data is mandatory and required for our internal procidures'),
        required=True,
    )

    country = schema.TextLine(
        title=_(u'Country'),
        description=_(u'This data is mandatory and required for our internal procidures'),
        required=True,
    )

    email = schema.TextLine(
        title=_(u'E-Mail'),
        description=_(u'We need this mandatory data to get in contact with you, if we have any questions'),
        required=True,
    )

    organisation = schema.TextLine(
        title=_(u'Organisation'),
        required=False,
    )

    registrationpayed = schema.Choice(
        title=_(u'Payment of the Registration Fee'),
        description=_(u'Have you already paid the registration fee?'),
        vocabulary=yesnochoice,
        required=True,
    )

    directives.widget(paymentway=RadioFieldWidget)
    paymentway = schema.List(
        title=_(u'Way of Registration Fee Payment'),
        description=_(
            u'If you already payed the registration fee, please tell us, which way you used to transfer the money.'),
        value_type=schema.Choice(source=vocabpaymentoptions),
        required=False,
    )

    usedbank = schema.TextLine(
        title=_(u'Used Bank Account'),
        description=_(
            u'If you transfered the Registration Fee via a bank account, please tell us the name and branch '
            u'of the bank you used. We need this information to identify your payment more quickly.'),
        required=False,
    )


class AttendeeAddForm(add.DefaultAddForm):
    portal_type = 'collective.conferences.attendee'

    def updateWidgets(self):
        super(AttendeeAddForm, self).updateWidgets()
        if getattr(self.context, 'conferencefee', None) == 0:
            self.widgets['registrationpayed'].mode = 'hidden'
            self.widgets['paymentway'].mode = 'hidden'
            self.widgets['usedbank'].mode = 'hidden'


class AttendeeAddView(add.DefaultAddView):
    form = AttendeeAddForm


class AttendeeEditForm(edit.DefaultEditForm):

    def updateWidgets(self):
        super(AttendeeEditForm, self).updateWidgets()
        parent = self.context.__parent__
        if getattr(parent, 'conferencefee', None) == 0:
            self.widgets['registrationpayed'].mode = 'hidden'
            self.widgets['paymentway'].mode = 'hidden'
            self.widgets['usedbank'].mode = 'hidden'


class AttendeeView(BrowserView):
    pass



# -*- coding: utf-8 -*-
from collective import dexteritytextindexer
from collective.conferences import _
from collective.conferences.common import endDefaultValue
from collective.conferences.common import startDefaultValue
from plone import api
from plone.app.contentlisting.interfaces import IContentListing
from plone.app.textfield import RichText
from plone.app.z3cform.widget import SelectFieldWidget
from plone.autoform import directives
from plone.autoform.directives import read_permission
from plone.autoform.directives import write_permission
from plone.namedfile.field import NamedBlobFile
from plone.supermodel import model
from plone.supermodel.directives import primary
from Products.CMFPlone.utils import safe_unicode
from Products.Five import BrowserView
from z3c.form.browser.radio import RadioFieldWidget
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope import schema
from zope.interface import directlyProvides
from zope.interface import Invalid
from zope.interface import invariant
from zope.schema.interfaces import IContextSourceBinder
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


# class StartBeforeEnd(Invalid):
#   __doc__ = _(u"The start or end date is invalid")


def vocabCfPTopics(context):
    # For add forms
    catalog = api.portal.get_tool(name='portal_catalog')
    results = catalog.uniqueValuesFor('callforpaper_topics')
    terms = []
    for value in results:
        terms.append(SimpleTerm(value, token=value.encode('unicode_escape'), title=value))

    return SimpleVocabulary(terms)


directlyProvides(vocabCfPTopics, IContextSourceBinder)


class ChooseLicense(Invalid):
    __doc__ = _(safe_unicode(
        'Please choose a license for your talk.'))


class ITalk(model.Schema):
    """A conference talk. Talks are managed inside tracks of the Program.
    """

    title = schema.TextLine(
        title=_(u'Title'),
        description=_(u'Talk title'),
    )

    description = schema.Text(
        title=_(u'Talk summary'),
    )

    primary('details')
    details = RichText(
        title=_(u'Talk details'),
        required=True,
    )

    speaker = RelationList(
        title=_(u'Presenter'),
        default=[],
        value_type=RelationChoice(vocabulary='ConferenceSpeaker'),
        required=False,
        missing_value=[],
    )
    directives.widget(
        'speaker',
        SelectFieldWidget,
    )

    dexteritytextindexer.searchable('call_for_paper_topics')
    directives.widget(call_for_paper_topic=RadioFieldWidget)
    call_for_paper_topic = schema.List(
        title=_(u'Choose the topic for your talk'),
        value_type=schema.Choice(source=vocabCfPTopics),
        required=True,
    )

    directives.widget(planedtalklength=RadioFieldWidget)
    planedtalklength = schema.List(
        title=_(u'Planed Length'),
        description=_(u"Give an estimation about the time you'd plan for your talk."),
        value_type=schema.Choice(source='TalkLength'),
        required=True,
    )

    directives.widget(license=RadioFieldWidget)
    license = schema.List(
        title=_(u'License Of Your Talk'),
        description=_(u'Choose a license for your talk'),
        value_type=schema.Choice(source='ContentLicense'),
        required=True,
    )

    messagetocommittee = schema.Text(
        title=_(u'Messages to the Program Committee'),
        description=_(u'You can give some information to the committee here, e.g. about days you are (not) '
                      u'available to give the talk'),
        required=False,
    )

    read_permission(conferencetrack='cmf.ReviewPortalContent')
    write_permission(conferencetrack='cmf.ReviewPortalContent')
    conferencetrack = RelationList(
        title=_(u'Choose the track for this talk'),
        default=[],
        value_type=RelationChoice(vocabulary='ConferenceTrack'),
        required=False,
        missing_value=[],
    )
    directives.widget(
        'conferencetrack',
        RadioFieldWidget,
    )

    read_permission(talklength='cmf.ReviewPortalContent')
    write_permission(talklength='cmf.ReviewPortalContent')
    directives.widget(talklength=RadioFieldWidget)
    talklength = schema.List(
        title=_(u'Talk Length'),
        description=_(u'Set a time frame for the talk in minutes.'),
        value_type=schema.Choice(source='TalkLength'),
        required=False,
    )

    read_permission(positionintrack='cmf.ReviewPortalContent')
    write_permission(positionintrack='cmf.ReviewPortalContent')
    positionintrack = schema.Int(
        title=_(u'Position In The Track'),
        description=_(u'Choose a number for the order in the track'),
        required=False,
    )

    write_permission(startitem='collective.conferences.ModifyTalktime')
    startitem = schema.Datetime(
        title=_(u'Startdate'),
        description=_(u'Start date'),
        defaultFactory=startDefaultValue,
        required=False,
    )

    write_permission(enditem='collective.conferences.ModifyTalktime')
    enditem = schema.Datetime(
        title=_(u'Enddate'),
        description=_(u'End date'),
        defaultFactory=endDefaultValue,
        required=False,
    )

    slides = NamedBlobFile(
        title=_(u'Presentation slides in ODT-File-Format'),
        description=_(u'Please upload your presentation shortly after you have given your talk.'),
        required=False,
    )

    slides2 = NamedBlobFile(
        title=_(u'Presentation slides in PDF-File-Format or PDF-Hybrid-File-Format'),
        description=_(u'Please upload your presentation shortly after you have given your talk.'),
        required=False,
    )

    slides3 = schema.URI(
        title=_(u'Link to the presentation slides in ODT-File-Format'),
        required=False,
    )

    slides4 = schema.URI(
        title=_(u'Link to the presentation slides in PDF-File-Format or PDF-Hybrid-File-Format'),
        required=False,
    )

    files = NamedBlobFile(
        title=_(u'Additional Files of your presentation.'),
        description=_(u'Please upload the additional files of your presentation (in archive format) '
                      u'shortly after you have given your talk.'),
        required=False,
    )

    files2 = schema.URI(
        title=_(u'Link to additional Files of your presentation in archive file format (e.g. zip-file-format).'),
        required=False,
    )

    video = schema.URI(
        title=_(u'Link to the Video of the talk'),
        required=False,
    )

    read_permission(reviewNotes='cmf.ReviewPortalContent')
    write_permission(reviewNotes='cmf.ReviewPortalContent')
    reviewNotes = schema.Text(
        title=u'Review notes',
        required=False,
    )

    @invariant
    def validateLicensechoosen(data):
        if not data.license:
            raise ChooseLicense(
                _(safe_unicode('Please choose a license for your talk.'),
                  ),
            )


def newtalkadded(self, event):
    if api.portal.get_registry_record(
            'plone.email_from_address') is not None:
        contactaddress = api.portal.get_registry_record(
            'plone.email_from_address')
        current_user = api.user.get_current()

        length = self.planedtalklength[0]
        cfp = self.call_for_paper_topic[0]
        details = self.details.output

        api.portal.send_email(
            recipient=current_user.getProperty('email'),
            sender=contactaddress,
            subject=safe_unicode('Your Talk Proposal'),
            body=safe_unicode('You submitted a conference talk:\n'
                              'title: {0},\n'
                              'summary: {1},\n'
                              'details: {2},\n'
                              'proposed length: {3} minutes\nfor the call for papers '
                              'topic: {4}\nwith the following message to the conference '
                              'committee: {5}\n\n'
                              'Best regards,\n'
                              'The Conference Committee').format(
                self.title, self.description, details,
                length, cfp, self.messagetocommittee),
        )


class TalkView(BrowserView):

    def canRequestReview(self):
        return api.user.has_permission('cmf.RequestReview', obj=self.context)

    def talkPresenters(self):
        results = []
        for rel in self.context.speaker:
            if rel.isBroken():
                # skip broken relations
                continue
            obj = rel.to_object
            if api.user.has_permission('View', obj=obj):
                results.append(obj)
        return IContentListing(results)

    def conferenceTrack(self):
        results = []
        for rel in self.context.conferencetrack:
            if rel.isBroken():
                # skip broken relations
                continue
            obj = rel.to_object
            if api.user.has_permission('View', obj=obj):
                results.append(obj)
        return IContentListing(results)



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from collective.conferences import _
from collective.conferences.common import endDefaultValue
from collective.conferences.common import startDefaultValue
from collective.conferences.track import ITrack
from plone import api
from plone.app.textfield import RichText
from plone.app.z3cform.widget import AjaxSelectFieldWidget
from plone.autoform import directives
from plone.supermodel import model
from plone.supermodel.directives import primary
from Products.Five import BrowserView
from zope import schema
from zope.interface import Invalid
from zope.interface import invariant


class StartBeforeEnd(Invalid):
    __doc__ = _(u'The start or end date is invalid')


class IProgram(model.Schema):
    """A conference program. Programs can contain Tracks.
    """

    title = schema.TextLine(
        title=_(u'Program name'),
    )

    description = schema.Text(
        title=_(u'Program summary'),
    )

    start = schema.Datetime(
        title=_(u'Start date'),
        required=False,
        defaultFactory=startDefaultValue,
    )

    end = schema.Datetime(
        title=_(u'End date'),
        required=False,
        defaultFactory=endDefaultValue,
    )

    primary('details')
    details = RichText(
        title=_(u'Details'),
        description=_(u'Details about the program'),
        required=False,
    )

    directives.widget(organizer=AjaxSelectFieldWidget)
    organizer = schema.Choice(
        title=_(u'Organiser'),
        vocabulary=u'plone.app.vocabularies.Users',
        required=False,
    )

    @invariant
    def validateStartEnd(data):
        if data.start is not None and data.end is not None:
            if data.start > data.end:
                raise StartBeforeEnd(_(
                    u'The start date must be before the end date.'))


# Views

class ProgramView(BrowserView):

    def tracks(self):
        """Return a catalog search result of tracks to show
        """

        context = aq_inner(self.context)
        catalog = api.portal.get_tool(name='portal_catalog')

        return catalog(object_provides=ITrack.__identifier__,
                       path='/'.join(context.getPhysicalPath()),
                       sort_order='sortable_title')


class FullprogramView(BrowserView):
    pass



# -*- coding: utf-8 -*-
from collective.conferences.testing import COLLECTIVE_CONFERENCES_INTEGRATION_TESTING
from collective.conferences.track import ITrack
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.dexterity.interfaces import IDexterityFTI
from zope.component import createObject
from zope.component import queryUtility

import unittest


class TestTrackIntegration(unittest.TestCase):
    layer = COLLECTIVE_CONFERENCES_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        portal = api.portal.get()
        api.content.create(type='collective.conferences.program', title='test-folder', container=portal)
        setRoles(self.portal, TEST_USER_ID, ['Member'])
        self.folder = self.portal['test-folder']

    def test_adding(self):
        portal = api.portal.get()
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        api.content.create(type='collective.conferences.program', title='program1', container=portal)
        p1 = portal['program1']
        api.content.create(type='collective.conferences.track', title='track1', container=p1)
        s1 = p1['track1']
        self.assertTrue(ITrack.providedBy(s1))

    def test_fti(self):
        fti = queryUtility(IDexterityFTI, name='collective.conferences.track')
        self.assertNotEqual(None, fti)

    def test_schema(self):
        fti = queryUtility(IDexterityFTI, name='collective.conferences.track')
        schema = fti.lookupSchema()
        self.assertEqual(ITrack, schema)

    def test_factory(self):
        fti = queryUtility(IDexterityFTI, name='collective.conferences.track')
        factory = fti.factory
        new_object = createObject(factory)
        self.assertTrue(ITrack.providedBy(new_object))

    def test_catalog_index_metadata(self):
        self.assertTrue('conferencetrack' in self.portal.portal_catalog.indexes())
        self.assertTrue('track' in self.portal.portal_catalog.schema())


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)



# -*- coding: utf-8 -*-
from collective.conferences import _
from plone.app.textfield import RichText
from plone.supermodel import model
from plone.supermodel.directives import primary
from Products.Five import BrowserView
from zope import schema


class ICallforpaper(model.Schema):
    """A call for paper for a conferences.
    A call for paper can contain incomming talks.
    """
    title = schema.TextLine(
        title=_(u'Call for paper title'),
    )

    description = schema.Text(
        title=_(u'Call for paper summary'),
        required=False,
    )

    primary('details')
    details = RichText(
        title=_(u'Details'),
        description=_(u'Details about the program'),
        required=True,
    )

    cfp_topics = schema.List(title=_(u'Topics for the Call for Papers'),
                             default=['Development',
                                      'Documentation',
                                      'Project-Administration'],
                             value_type=schema.TextLine(),
                             )


# Views

class CallforpaperView(BrowserView):
    pass



# -*- coding: utf-8 -*-
from collective.conferences import _
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary

import datetime


yesnochoice = SimpleVocabulary(
    [SimpleTerm(value=0, title=_(u'No')),
     SimpleTerm(value=1, title=_(u'Yes'))],
)


def startDefaultValue():
    return datetime.datetime.today() + datetime.timedelta(14)


def endDefaultValue():
    return datetime.datetime.today() + datetime.timedelta(17)



# -*- coding: utf-8 -*-
from collective.conferences.testing import COLLECTIVE_CONFERENCES_INTEGRATION_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID

import unittest


class TestPresenterIntegration(unittest.TestCase):
    layer = COLLECTIVE_CONFERENCES_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        portal = api.portal.get()
        api.content.create(type='collective.conferences.speakerfolder', title='test-folder', container=portal)
        setRoles(self.portal, TEST_USER_ID, ['Member'])
        self.folder = self.portal['test-folder']


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)



# -*- coding: utf-8 -*-
from plone import api
from plone.app.vocabularies.terms import safe_simplevocabulary_from_values
from zope.interface import implementer
from zope.interface import provider
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


@provider(IVocabularyFactory)
def BreakLengthVocabularyFactory(context):
    values = api.portal.get_registry_record('collectiveconf.break_length')
    return safe_simplevocabulary_from_values(values)


@provider(IVocabularyFactory)
def TalkLengthVocabularyFactory(context):
    values = api.portal.get_registry_record('collectiveconf.talk_length')
    return safe_simplevocabulary_from_values(values)


@provider(IVocabularyFactory)
def WorkshopLengthVocabularyFactory(context):
    values = api.portal.get_registry_record('collectiveconf.workshop_length')
    return safe_simplevocabulary_from_values(values)


@provider(IVocabularyFactory)
def LicenseVocabularyFactory(context):
    values = api.portal.get_registry_record('collectiveconf.license')
    return safe_simplevocabulary_from_values(values)


@implementer(IVocabularyFactory)
class ConferenceSpeakerVocabulary(object):

    def __call__(self, context=None):
        terms = []
        # Use getObject since the DataConverter expects a real object.
        for brain in api.content.find(portal_type='collective.conferences.conferencespeaker', sort_on='sortable_title'):
            terms.append(SimpleTerm(
                value=brain.getObject(),
                token=brain.UID,
                title=u'{0} ({1})'.format(brain.Title, brain.getPath()),
            ))
        return SimpleVocabulary(terms)


ConferenceSpeakerVocabularyFactory = ConferenceSpeakerVocabulary()


@implementer(IVocabularyFactory)
class ConferenceRoomVocabulary(object):

    def __call__(self, context=None):
        terms = []
        # Use getObject since the DataConverter expects a real object.
        for brain in api.content.find(portal_type='collective.conferences.room', sort_on='sortable_title'):
            terms.append(SimpleTerm(
                value=brain.getObject(),
                token=brain.UID,
                title=u'{0} ({1})'.format(brain.Title, brain.getPath()),
            ))
        return SimpleVocabulary(terms)


ConferenceRoomVocabularyFactory = ConferenceRoomVocabulary()


@implementer(IVocabularyFactory)
class ConferenceTrackVocabulary(object):

    def __call__(self, context=None):
        terms = []
        # Use getObject since the DataConverter expects a real object.
        for brain in api.content.find(portal_type='collective.conferences.track', sort_on='sortable_title'):
            terms.append(SimpleTerm(
                value=brain.getObject(),
                token=brain.UID,
                title=u'{0} ({1})'.format(brain.Title, brain.getPath()),
            ))
        return SimpleVocabulary(terms)


ConferenceTrackVocabularyFactory = ConferenceTrackVocabulary()



# -*- coding: utf-8 -*-
from plone.app.content.interfaces import INameFromTitle
from zope.interface import implementer


class ICustomSpeakerURL(INameFromTitle):
    def title():
        """Return a processed title"""


@implementer(ICustomSpeakerURL)
class CustomSpeakerURL(object):

    def __init__(self, context):
        self.context = context

    @property
    def title(self):
        return self.context.lastname



# -*- coding: utf-8 -*-
from collective.conferences import _
from collective.conferences.common import endDefaultValue
from collective.conferences.common import startDefaultValue
from plone import api
from plone.app.textfield import RichText
from plone.autoform import directives
from plone.autoform.directives import write_permission
from plone.dexterity.browser.view import DefaultView
from plone.supermodel import model
from plone.supermodel.directives import primary
from z3c.form.browser.radio import RadioFieldWidget
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope import schema


class IConferencebreak(model.Schema):
    title = schema.TextLine(
        title=_(u'Title'),
        description=_(u'Conference break title'),
    )

    description = schema.Text(
        title=_(u'Conference break summary'),
        required=False,
    )

    primary('details')
    details = RichText(
        title=_(u'Conference break details'),
        required=False,
    )

    write_permission(startitem='collective.conferences.ModifyTalktime')
    startitem = schema.Datetime(
        title=_(u'Startdate'),
        description=_(u'Start date'),
        defaultFactory=startDefaultValue,
        required=False,
    )

    write_permission(enditem='collective.conferences.ModifyTalktime')
    enditem = schema.Datetime(
        title=_(u'Enddate'),
        description=_(u'End date'),
        defaultFactory=endDefaultValue,
        required=False,
    )
    write_permission(breaklength='collective.conferences.ModifyTalktime')
    directives.widget(breaklength=RadioFieldWidget)
    breaklength = schema.List(
        title=_(u'Length'),
        value_type=schema.Choice(source='BreakLength'),
        required=True,
    )

    write_permission(conferencetrack='collective.conferences.ModifyTalktime')
    conferencetrack = RelationList(
        title=_(u'Choose the track for this break'),
        default=[],
        value_type=RelationChoice(vocabulary='ConferenceTrack'),
        required=False,
        missing_value=[],
    )
    directives.widget(
        'conferencetrack',
        RadioFieldWidget,
    )

    positionintrack = schema.Int(
        title=_(u'Position In The Track'),
        description=_(u'Choose a number for the order in the track'),
        required=False,
    )


class ConferencebreakView(DefaultView):

    def canRequestReview(self):
        return api.user.has_permission('cmf.RequestReview', obj=self.context)



# -*- coding: utf-8 -*-
from collective.conferences import _
from plone.autoform import directives
from plone.supermodel import model
from Products.Five import BrowserView
from zope import schema


class ITalksfolder(model.Schema):

    directives.mode(title='hidden')
    title = schema.TextLine(
        title=_(u'Name Of The Folder For Talks'),
        default=_(u'Talks'),
    )

    description = schema.Text(
        title=_(u'Talk Folder Description'),
        required=False,
    )


class TalksfolderView(BrowserView):

    pass



# -*- coding: utf-8 -*-
from collective.conferences import _
from plone import api
from plone.supermodel import model
from Products.Five import BrowserView
from zope import schema


class ISpeakerfolder(model.Schema):
    """A speaker folder. The speaker of the conference are created in the folder.
    """

    title = schema.TextLine(
        title=_(u'Name of the speaker folder'),
    )

    description = schema.Text(
        title=_(u'speakerfolder description'),
        required=False,
    )


class SpeakerfolderView(BrowserView):

    def canRequestReview(self):
        return api.user.has_permission('cmf.RequestReview', obj=self.context)



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from collective.conferences import _
from plone import api
from plone.app.textfield import RichText
from plone.app.z3cform.widget import SelectFieldWidget
from plone.autoform.form import AutoExtensibleForm
from plone.formwidget.recaptcha.widget import ReCaptchaFieldWidget
from plone.z3cform.layout import wrap_form
from Products.CMFPlone.utils import safe_unicode
from z3c.form import button
from z3c.form import field
from z3c.form import form
from z3c.form.browser.radio import RadioFieldWidget
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope import interface
from zope import schema
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.interface import directlyProvides
from zope.interface import implementer
from zope.interface import Invalid
from zope.interface import invariant
from zope.schema.interfaces import IContextSourceBinder
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary

import logging


logger = logging.getLogger(__name__)


def vocabCfPTopics(context):
    # For add forms
    catalog = api.portal.get_tool(name='portal_catalog')
    results = catalog.uniqueValuesFor('callforpaper_topics')
    terms = []
    for value in results:
        terms.append(SimpleTerm(value, token=value.encode('unicode_escape'), title=value))

    return SimpleVocabulary(terms)


directlyProvides(vocabCfPTopics, IContextSourceBinder)


class ChooseLicense(Invalid):
    __doc__ = _(safe_unicode(
        'Please choose a license for your talk.'))


class IReCaptchaForm(interface.Interface):
    captcha = schema.TextLine(
        title=safe_unicode('ReCaptcha'),
        description=safe_unicode(''),
        required=False,
    )


class ReCaptcha(object):
    captcha = safe_unicode('')

    def __init__(self, context):
        self.context = context


class NewWorkshopSchema(interface.Interface):
    workshoptitle = schema.TextLine(
        title=_(u'The Title Of Your Workshop'),
        description=_(u'Fill in the title of your proposed conference workshop'),
    )

    workshopdescription = schema.Text(
        title=_(u'Workshop summary'),
    )

    workshopdetails = RichText(
        title=_(u'Workshop details'),
        required=True,
    )

    speaker = RelationList(
        title=_(u'Workshop Leader'),
        default=[],
        value_type=RelationChoice(vocabulary='ConferenceSpeaker'),
        required=False,
        missing_value=[],
    )

    cfp_topic = schema.List(
        title=_(u'Choose the topic for your workshop'),
        value_type=schema.Choice(source=vocabCfPTopics),
        required=True,
    )

    wtalklength = schema.List(
        title=_(u'Planed Length'),
        description=_(u"Give an estimation about the time you'd plan for your workshop."),
        value_type=schema.Choice(source='WorkshopLength'),
        required=True,
    )

    license = schema.List(
        title=_(u'License Of Your Talk'),
        description=_(u'Choose a license for your talk'),
        value_type=schema.Choice(source='ContentLicense'),
        required=True,
    )

    messagetocommittee = schema.Text(
        title=_(u'Messages to the Program Committee'),
        description=_(
            u'You can give some information to the committee here, e.g. about days you are (not) available '
            u'to give the workshop'),
        required=False,
    )

    @invariant
    def validateLicensechoosen(data):
        if not data.license:
            raise ChooseLicense(
                _(safe_unicode('Please choose a license for your talk.'),
                  ),
            )


@implementer(NewWorkshopSchema)
@adapter(interface.Interface)
class NewWorkshopSchemaAdapter(object):

    def __init__(self, context):
        self.workshoptitle = None
        self.workshopdescription = None
        self.workshopdetails = None
        self.speaker = None
        self.cfp_topic = None
        self.wtalklength = None
        self.messagetocommittee = None


class NewWorkshopForm(AutoExtensibleForm, form.Form):
    schema = NewWorkshopSchema
    form_name = 'newworkshopform'
    label = _(safe_unicode('Submit A Conference Workshop'))
    description = _(safe_unicode('Submit a proposal for a conference workshop.'))

    fields = field.Fields(NewWorkshopSchema, IReCaptchaForm)
    fields['captcha'].widgetFactory = ReCaptchaFieldWidget
    fields['cfp_topic'].widgetFactory = RadioFieldWidget
    fields['wtalklength'].widgetFactory = RadioFieldWidget
    fields['speaker'].widgetFactory = SelectFieldWidget
    fields['license'].widgetFactory = RadioFieldWidget

    def update(self):
        # disable Plone's editable border
        self.request.set('disable_border', True)

        # call the base class version - this is very important!
        super(NewWorkshopForm, self).update()

    @button.buttonAndHandler(_(safe_unicode('Submit Your Workshop')))
    def handleApply(self, action):
        data, errors = self.extractData()
        captcha = getMultiAdapter(
            (aq_inner(self.context), self.request),
            name='recaptcha',
        )

        if errors:
            self.status = self.formErrorsMessage
            return

        elif captcha.verify():
            logger.info('ReCaptcha validation passed.')
        else:
            logger.info(
                'Please validate the recaptcha field before sending the form.',
            )
            api.portal.show_message(
                message=_(
                    safe_unicode('Please validate the recaptcha field '
                                 'before sending the form.')),
                request=self.request,
                type='error')
            return

        portal = api.portal.get()
        api.content.create(
            type='collective.conferences.workshop',
            title=data['workshoptitle'],
            description=data['workshopdescription'],
            details=data['workshopdetails'],
            call_for_paper_topic=data['cfp_topic'],
            planedworkshoplength=data['wtalklength'],
            messagetocommittee=data['messagetocommittee'],
            container=portal['workshops'],
        )

        if api.portal.get_registry_record(
                'plone.email_from_address') is not None:
            contactaddress = api.portal.get_registry_record(
                'plone.email_from_address')
            current_user = api.user.get_current()
            length = (data['wtalklength'])[0]
            cfp = (data['cfp_topic'])[0]
            details = (data['workshopdetails']).output

            api.portal.send_email(
                recipient=current_user.getProperty('email'),
                sender=contactaddress,
                subject=safe_unicode('Your Workshop Proposal'),
                body=safe_unicode('You submitted a conference workshop:\n'
                                  'title: {0},\nsummary: {1},\ndetails: {2},\n'
                                  'proposed length: {3} minutes\nfor the call for papers '
                                  'topic: {4}\nwith the following message to the conference '
                                  'committee: {5}').format(
                    data['workshoptitle'], data['workshopdescription'], details,
                    length, cfp, data['messagetocommittee']),
            )

        api.portal.show_message(
            message=_(safe_unicode('The workshop has been submitted.')),
            request=self.request,
            type='info')

        contextURL = self.context.absolute_url()
        self.request.response.redirect(contextURL)

    @button.buttonAndHandler(_(safe_unicode('Cancel')))
    def handleCancel(self, action):
        """User cancelled. Redirect back to the front page.
            """
        contextURL = self.context.absolute_url()
        self.request.response.redirect(contextURL)


ReCaptchaForm = wrap_form(NewWorkshopForm)



# -*- coding: utf-8 -*-
from collective.conferences.program import endDefaultValue
from collective.conferences.program import IProgram
from collective.conferences.program import startDefaultValue
from collective.conferences.testing import COLLECTIVE_CONFERENCES_INTEGRATION_TESTING
from DateTime import DateTime
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.dexterity.interfaces import IDexterityFTI
from zope.component import createObject
from zope.component import queryUtility

import datetime
import unittest


class MockProgram(object):
    pass


class TestProgramUnit(unittest.TestCase):

    def test_start_defaults(self):
        default_value = startDefaultValue()
        today = datetime.datetime.today()
        delta = default_value - today
        self.assertEqual(13, delta.days)

    def test_end_default(self):
        default_value = endDefaultValue()
        today = datetime.datetime.today()
        delta = default_value - today
        self.assertEqual(16, delta.days)


class TestProgramIntegration(unittest.TestCase):
    layer = COLLECTIVE_CONFERENCES_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        portal = api.portal.get()
        api.content.create(type='Folder', title='test-folder', container=portal)
        setRoles(self.portal, TEST_USER_ID, ['Member'])
        self.folder = self.portal['test-folder']

    def test_adding(self):
        portal = api.portal.get()
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        api.content.create(type='collective.conferences.program', title='program1', container=portal)
        p1 = portal['program1']
        self.assertTrue(IProgram.providedBy(p1))

    def test_fti(self):
        fti = queryUtility(IDexterityFTI, name='collective.conferences.program')
        self.assertNotEqual(None, fti)

    def test_schema(self):
        fti = queryUtility(IDexterityFTI, name='collective.conferences.program')
        schema = fti.lookupSchema()
        self.assertEqual(IProgram, schema)

    def test_factory(self):
        fti = queryUtility(IDexterityFTI, name='collective.conferences.program')
        factory = fti.factory
        new_object = createObject(factory)
        self.assertTrue(IProgram.providedBy(new_object))

    def test_view(self):
        portal = api.portal.get()
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        api.content.create(type='collective.conferences.program', title='program1', container=portal)
        p1 = portal['program1']
        view = p1.restrictedTraverse('@@view')
        tracks = view.tracks()
        self.assertEqual(0, len(tracks))

    def test_start_end_dates_indexed(self):
        portal = api.portal.get()
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        api.content.create(type='collective.conferences.program', title='program1', container=portal)
        p1 = portal['program1']
        p1.start = datetime.datetime(2009, 1, 1, 14, 00)
        p1.end = datetime.datetime(2009, 1, 2, 15, 00)
        p1.reindexObject()

        result = self.portal.portal_catalog(path='/'.join(p1.getPhysicalPath()))

        self.assertEqual(1, len(result))
        self.assertEqual(DateTime(result[0].start), DateTime('2009-01-01T14:00:00').toZone('UTC'))
        self.assertEqual(DateTime(result[0].end), DateTime('2009-01-02T15:00:00').toZone('UTC'))


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)



# -*- coding: utf-8 -*-
from plone import api
from Products.CMFPlone.interfaces import INonInstallable
from zope.interface import implementer

import logging


PROFILE_ID = 'profile-collective.conferences:default'

# catalog indexes to add
CATALOG_INDEXES = (

)


@implementer(INonInstallable)
class HiddenProfiles(object):

    def getNonInstallableProfiles(self):
        """Hide uninstall profile from site-creation and quickinstaller."""
        return [
            'collective.conferences:uninstall',
        ]


def post_install(context):
    """Post install script"""
    # Do something at the end of the installation of this package.
    import_catalog_indexes(context)


def uninstall(context):
    """Uninstall script"""
    # Do something at the end of the uninstallation of this package.


def import_catalog_indexes(context, logger=None):
    """Method to add our wanted indexes to the portal_catalog.

    @parameters:

    When called from the import_various method below, 'context' is
    the plone site and 'logger' is the portal_setup logger.  But
    this method can also be used as upgrade step, in which case
    'context' will be portal_setup and 'logger' will be None.
    """

    if logger is None:
        # Called as upgrade step: define our own logger.
        logger = logging.getLogger('vncportal.policy')

    # Run the catalog.xml step as that may have defined new metadata
    # columns.  We could instead add <depends name="catalog"/> to
    # the registration of our import step in zcml, but doing it in
    # code makes this method usable as upgrade step as well.
    # Remove these lines when you have no catalog.xml file.
    setup = api.portal.get_tool('portal_setup')
    setup.runImportStepFromProfile(PROFILE_ID, 'catalog')
    catalog = api.portal.get_tool('portal_catalog')

    indexes = catalog.indexes()
    indexables = []

    for name, meta_type in CATALOG_INDEXES:
        if name not in indexes:
            catalog.addIndex(name, meta_type)
            indexables.append(name)
            logger.info('Added %s for field %s.', meta_type, name)

    if len(indexables) > 0:
        logger.info('Indexing new indexes %s.', ', '.join(indexables))
        catalog.manage_reindexIndex(ids=indexables)



# -*- coding: utf-8 -*-
"""Installer for the collective.conferences package."""

from setuptools import find_packages
from setuptools import setup


long_description = '\n\n'.join([
    open('README.rst').read(),
    open('CONTRIBUTORS.rst').read(),
    open('CHANGES.rst').read(),
])

version = '0.2.dev0'


setup(
    name='collective.conferences',
    version='0.1.dev0',
    description="A Conference Management System for Plone",
    long_description=long_description,
    # Get more strings from
    # http://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: Addon",
        "Framework :: Plone :: 5.2",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        ],
    keywords='Python Plone Conference Organisation Tool',
    author='Andreas Mantke',
    author_email='maand@gmx.de',
    url='http://github.com/collective/collective.conferences',
    project_urls={
        'PyPI': 'https://pypi.python.org/pypi/collective.conferences',
        'Source': 'https://github.com/collective/collective.conferences',
        'Tracker': 'https://github.com/collective/collective.conferences/issues',
        # 'Documentation':
        # 'https://collective.addons.readthedocs.io/en/latest/',
    },
    license='GPL version 2',
    packages=find_packages('src', exclude=['ez_setup']),
    namespace_packages=['collective'],
    package_dir={'': 'src'},
    include_package_data=True,
    zip_safe=False,
    python_requires="==2.7, >=3.6",
    install_requires=[
        'setuptools',
        # -*- Extra requirements: -*-
        'z3c.jbot',
        'plone.api>=1.8.4',
        'plone.restapi',
        'plone.app.dexterity',
        'Products.validation',
        'collective.dexteritytextindexer',
        'plone.formwidget.autocomplete',
        'plone.formwidget.contenttree',
        'plone.formwidget.recaptcha',
        'plone.app.z3cform >= 3.2.0',
      ],
    extras_require={
        'test': [
            'plone.app.testing',
            # Plone KGS does not use this version, because it would break
            # Remove if your package shall be part of coredev.
            # plone_coredev tests as of 2016-04-01.
            'plone.testing>=5.0.0',
            'plone.app.contenttypes',
            'plone.app.robotframework[debug]',
            'Products.validation',
            'plone.formwidget.recaptcha',
        ],
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    [console_scripts]
    update_locale = collective.conferences.locales.update:update_locale
    """,
)



# -*- coding: utf-8 -*-
from plone.app.registry.browser.controlpanel import ControlPanelFormWrapper
from plone.app.registry.browser.controlpanel import RegistryEditForm
from plone.z3cform import layout
from zope import schema
from zope.interface import Interface


class ICollectiveconfControlPanel(Interface):
    break_length = schema.Tuple(
        title=u'Length Of Conference Breaks',
        description=u'Fill in the time slots for conference breaks in minutes. Use a new line for every '
                    u"value / talk length. Write only the numbers without the addition 'minutes'.",
        value_type=schema.TextLine(),
        required=True,
    )

    talk_length = schema.Tuple(
        title=u'Length Of Talks',
        description=u'Fill in the time slots for conference talks in minutes. Use a new line for every '
                    u"value / talk length. Write only the numbers without the addition 'minutes'.",
        value_type=schema.TextLine(),
        required=True,
    )

    workshop_length = schema.Tuple(
        title=u'Length Of Workshops',
        description=u'Fill in the time slots for workshops at the conference in minutes. Use a new line for every '
                    u"value / workshop length. Write only the numbers without the addition 'minutes'.",
        value_type=schema.TextLine(),
        required=True,
    )

    license = schema.Tuple(
        title=u'Licenses Of Talks / Workshops',
        description=u'Fill in the names of the licenses for conference talks and workshops. Use a new line for '
                    u'every value / license name.',
        value_type=schema.TextLine(),
        required=True,
    )


class CollectiveconfControlPanelForm(RegistryEditForm):
    schema = ICollectiveconfControlPanel
    schema_prefix = 'collectiveconf'
    label = u'Collectiveconf Settings'


CollectiveconfControlPanelView = layout.wrap_form(
    CollectiveconfControlPanelForm, ControlPanelFormWrapper)
